from setuptools import setup

setup(name='icanvas',
      version='0.0.25',
      description='Canvas Installation Module',
      url='http://github.com/sunlove123',
      author='DeenaDhayalan',
      author_email='deena.boopathy@intellectdesign.com',
      license='MIT',
      packages=['icanvas'],
      zip_safe=False)
